@@ -1,12 +1,12 @@
 #############################################################################
-# output the graph in VCG text
+# Output the graph as VCG or GDL text.
 #
 # (c) by Tels 2004-2007.
 #############################################################################
 
 package Graph::Easy::As_vcg;
 
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 #############################################################################
 #############################################################################
@@ -59,8 +59,6 @@ my $vcg_remap = {
     bordercolor => undef,
     borderstyle => undef,
     borderwidth => undef,
-    # XXX TODO map to edgeclass and graph.classname
-    class => undef,
     colorscheme => undef,
     end => undef,
     fontsize => undef,
@@ -142,17 +140,36 @@ sub _as_vcg
   # convert the graph to a textual representation
   # does not need a layout() beforehand!
 
+  # gather all edge classes to build the classname attribute from them:
+  $self->{_vcg_edge_classes} = {};
+  for my $e (values %{$self->{edges}})
+    {
+    my $class = $e->sub_class();
+    $self->{_vcg_edge_classes}->{$class} = undef if defined $class && $class ne '';
+    }
+  # sort gathered class names and map them to integers
+  my $class_names = '';
+  if (keys %{$self->{_vcg_edge_classes}} > 0)
+    {
+    my $i = 1;
+    $class_names = "\n  classname: ";
+    for my $ec (sort keys %{$self->{_vcg_edge_classes}})
+      {
+      $self->{_vcg_edge_classes}->{$ec} = $i;	# remember mapping
+      $class_names .= $i.":\"$ec\",";
+      $i++;
+      }
+    $class_names =~ s/,\z/\n/;		# remove last ","
+    }
+
   # generate the class attributes first
   my $label = $self->label();
   my $t = ''; $t = "\n  title: \"$label\"" if $label ne '';
 
   my $txt = "graph: {$t\n\n" .
             "  // Generated by Graph::Easy $Graph::Easy::VERSION" .
-	    " at " . scalar localtime() . "\n";
-
-  my $flow = $self->attribute('graph','flow');
-  $flow = 'east' unless defined $flow;
-  $flow = Graph::Easy->_direction_as_number($flow);
+	    " at " . scalar localtime() . "\n" .
+	    $class_names;
 
   my $groups = $self->groups();
 
@@ -167,30 +184,8 @@ sub _as_vcg
     {
     next if $class =~ /\./;		# skip subclasses
 
-    my $out = $self->_remap_attributes( $class, $atts->{$class}, $vcg_remap, 'noquote');
-
-    # per default, our nodes are rectangular, white, filled boxes
-    if ($class eq 'node')
-      {
-#      $out->{shape} = 'box' unless $out->{shape}; 
-#      $out->{style} = 'filled' unless $out->{style};
-#      $out->{fontsize} = '11' unless $out->{fontsize};
-#      $out->{fillcolor} = 'white' unless $out->{fillcolor};
-      }
-    elsif ($class eq 'graph')
-      {
-#      $out->{rankdir} = 'LR' if $flow == 90 || $flow == 270;
-#      $out->{labelloc} = 'top' if defined $out->{label} && !defined $out->{labelloc};
-#      $out->{style} = 'filled' if $groups > 0;
-      }
-    elsif ($class eq 'edge')
-      {
-#      $out->{dir} = 'back' if $flow == 270 || $flow == 0;
-#      my ($name,$style) = $self->_graphviz_remap_arrow_style('',
-#        $self->attribute('edge','arrowstyle') );
-#      $out->{$name} = $style;
-      }
-
+    # XXX TODO
+#    my $out = $self->_remap_attributes( $class, $atts->{$class}, $vcg_remap, 'noquote');
 #    $txt .= $self->_att_as_vcg($out);
     }
 
@@ -316,6 +311,7 @@ sub _as_vcg
     delete $n->{_p};
     }
   delete $self->{_vcg_invis};		# invisible helper nodes for joints
+  delete $self->{_vcg_edge_classes};
 
   $txt .  "\n}\n";			# close the graph
   }
@@ -367,6 +363,8 @@ sub attributes_as_vcg
     {
     $a->{sourcename} = $self->{from}->{name};
     $a->{targetname} = $self->{to}->{name};
+    my $class = $self->sub_class();
+    $a->{class} = $self->{graph}->{_vcg_edge_classes}->{ $class } if defined $class && $class ne '';
     }
   else
     {
@@ -433,7 +431,8 @@ sub attributes_as_vcg
     {
     my $v = $a->{$atr};
     $v =~ s/"/\\"/g;		# '2"' => '2\"'
-    $att .= "$atr: \"$v\" ";
+    $v = '"' . $v . '"' unless $v =~ /^[0-9]+\z/;	# 1, "1a"
+    $att .= "$atr: $v ";
     }
   $att =~ s/,\s$//;             # remove last ","
 
